/*
 * GenericAddItemsDialog.java
 *
 * Created on 26-may-2011, 18:53:29
 */
package scimat.gui.components.adddialog;

import java.util.ArrayList;
import scimat.gui.components.itemslist.GenericSelectManyItemsPanel;
import scimat.gui.components.observer.ElementsCountObserver;
import scimat.model.knowledgebase.exception.KnowledgeBaseException;
import scimat.project.observer.EntityObserver;

/**
 *
 * @author mjcobo
 */
public abstract class GenericAddItemsDialog<M extends Comparable<M>, I extends Comparable<I>> 
extends javax.swing.JDialog implements ElementsCountObserver, EntityObserver<I> {

  /** Creates new form GenericAddItemsDialog */
  public GenericAddItemsDialog(java.awt.Frame parent, 
          GenericSelectManyItemsPanel<I> manyToManyPanel) {
    super(parent, true);
    
    this.manyToManyPanel = manyToManyPanel;
    
    initComponents();
    
    this.itemsPanel.add(this.manyToManyPanel);
    this.manyToManyPanel.addTargetElementsCountObserver(this);
    
  }

  /**
   * 
   */
  public void reset() {
  
    this.masterItem = null;
    this.addButton.setEnabled(false);
    this.manyToManyPanel.refreshData(new ArrayList<I>());
  }
  
  /**
   * 
   * @param items 
   */
  public void refreshData(M masterItem, ArrayList<I> items) {
  
    this.masterItem = masterItem;
    this.manyToManyPanel.refreshData(items);
    this.addButton.setEnabled(false);
  }
  
  /**
   * 
   * @param masterItem
   * @param itemsToAdd 
   */
  public abstract void addAction(M masterItem, ArrayList<I> itemsToAdd);
  
  /**
   * 
   */
  public abstract void addNewItemAction();
  
  public abstract void setEntityObserver(boolean enabled);

  /**
   * 
   * @param items
   * @throws KnowledgeBaseException 
   */
  public void entityAdded(ArrayList<I> items) throws KnowledgeBaseException {
    
    this.manyToManyPanel.addNewTargetItems(items);
  }

  /**
   * 
   * @param items
   * @throws KnowledgeBaseException 
   */
  public void entityUpdated(ArrayList<I> items) throws KnowledgeBaseException {
    
  }

  /**
   * 
   * @throws KnowledgeBaseException 
   */
  public void entityRefresh() throws KnowledgeBaseException {
    
  }

  /**
   * 
   * @param items
   * @throws KnowledgeBaseException 
   */
  public void entityRemoved(ArrayList<I> items) throws KnowledgeBaseException {
    
  }
  
  /**
   * 
   * @param newElementsCount 
   */
  public void elementsCountChanged(int newElementsCount) {
    
    this.addButton.setEnabled(newElementsCount > 0);
  }

  /** This method is called from within the constructor to
   * initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is
   * always regenerated by the Form Editor.
   */
  @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        itemsPanel = new javax.swing.JPanel();
        separator = new javax.swing.JSeparator();
        addButton = new javax.swing.JButton();
        cancelButton = new javax.swing.JButton();
        newItemButton = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowActivated(java.awt.event.WindowEvent evt) {
                formWindowActivated(evt);
            }
            public void windowClosed(java.awt.event.WindowEvent evt) {
                formWindowClosed(evt);
            }
        });

        itemsPanel.setLayout(new javax.swing.BoxLayout(itemsPanel, javax.swing.BoxLayout.LINE_AXIS));

        addButton.setText("Add");
        addButton.setToolTipText("Add the selected items");
        addButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                addButtonActionPerformed(evt);
            }
        });

        cancelButton.setText("Cancel");
        cancelButton.setToolTipText("Cancel the action");
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelButtonActionPerformed(evt);
            }
        });

        newItemButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/images/add-icon-16x16.png"))); // NOI18N
        newItemButton.setToolTipText("New item");
        newItemButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                newItemButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(separator, javax.swing.GroupLayout.DEFAULT_SIZE, 380, Short.MAX_VALUE)
                    .addComponent(itemsPanel, javax.swing.GroupLayout.DEFAULT_SIZE, 380, Short.MAX_VALUE)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                        .addComponent(newItemButton)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 209, Short.MAX_VALUE)
                        .addComponent(addButton)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(cancelButton)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(itemsPanel, javax.swing.GroupLayout.DEFAULT_SIZE, 173, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(separator, javax.swing.GroupLayout.PREFERRED_SIZE, 10, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(addButton)
                        .addComponent(cancelButton))
                    .addComponent(newItemButton))
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

 
  /**
   * 
   * @param evt 
   */
  private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelButtonActionPerformed
    dispose();
  }//GEN-LAST:event_cancelButtonActionPerformed

  /**
   * 
   * @param evt 
   */
  private void addButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_addButtonActionPerformed
    
    addAction(this.masterItem, this.manyToManyPanel.getTargetItems());
  }//GEN-LAST:event_addButtonActionPerformed

  /**
   * 
   * @param evt 
   */
  private void newItemButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_newItemButtonActionPerformed
    addNewItemAction();
  }//GEN-LAST:event_newItemButtonActionPerformed

  /**
   * 
   * @param evt 
   */
  private void formWindowClosed(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosed
    setEntityObserver(false);
  }//GEN-LAST:event_formWindowClosed

  /**
   * 
   * @param evt 
   */
  private void formWindowActivated(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowActivated
    setEntityObserver(true);
  }//GEN-LAST:event_formWindowActivated

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton addButton;
    private javax.swing.JButton cancelButton;
    private javax.swing.JPanel itemsPanel;
    private javax.swing.JButton newItemButton;
    private javax.swing.JSeparator separator;
    // End of variables declaration//GEN-END:variables
  private M masterItem;
  private GenericSelectManyItemsPanel<I> manyToManyPanel;
}
