/*
 * GenericSelectOneItemPanel.java
 *
 * Created on 31-mar-2011, 23:31:38
 */
package scimat.gui.components.itemslist;

import java.util.ArrayList;
import javax.swing.ListSelectionModel;
import scimat.gui.components.detailspanel.GenericDetailPanel;
import scimat.gui.components.observer.SelectionObserver;
import scimat.gui.components.observer.TargetItemSelectedObserver;

/**
 *
 * @author mjcobo
 * 
 * NOTE: the Entity of the source and selected elements is different because 
 * they can need show the information with differnet views.
 * 
 * @param <E> The entity of the source elements. It can be represented by a 
 *             View
 */
public class GenericSelectOneItemPanel<E extends Comparable<E>>
                                       extends javax.swing.JPanel 
                                       implements SelectionObserver {

  /** Creates new form GenericSelectOneItemPanel */
  public GenericSelectOneItemPanel(GenericDynamicItemsListPanel<E> sourceListPanel, 
          GenericDetailPanel<E> detailItemPanel) {
    
    this.sourceListPanel = sourceListPanel;
    this.detailItemPanel = detailItemPanel;
    
    initComponents();
    
    this.sourceListPanel.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    this.sourceListPanel.addSelectionObserver(this);
    this.sourcePanel.add(this.sourceListPanel);
    this.detailPanel.add(this.detailItemPanel);
  }
  
  /**
   *
   * @param o
   */
  public void addTargetItemSelectedObserver(TargetItemSelectedObserver o) {

    this.targetItemSelecteds.add(o);
  }
  
  /**
   * 
   */
  private void notifyTargetItemSelectedObserver() {
  
    int i;
    boolean flag = this.selectedItem != null;
    
    for (i = 0; i < this.targetItemSelecteds.size(); i++) {
    
      this.targetItemSelecteds.get(i).targetItemSelectionChanged(flag);
    }
    
  }
  
  /**
   * 
   */
  public void reset() {
  
    this.selectedItem = null;
    
    this.sourceListPanel.refreshItems(new ArrayList<E>());
    
    this.detailItemPanel.refreshItem(selectedItem);
    
    notifyTargetItemSelectedObserver();
  }
  
  /**
   * 
   */
  public void refreshSourceItems(ArrayList<E> items) {
  
    this.sourceListPanel.refreshItems(items);
    this.detailItemPanel.refreshItem(null);
  }

  /**
   * 
   * @param selection 
   */
  public void selectionChangeHappened(int[] selection) {
    
    if (selection.length == 1) {
      
      this.selectedItem = this.sourceListPanel.getItem(selection[0]);

    } else {

      this.selectedItem = null;
    }

    this.detailItemPanel.refreshItem(selectedItem);

    notifyTargetItemSelectedObserver();
  }

  /**
   * 
   * @return 
   */
  public E getSelectedItem() {
    return selectedItem;
  }
  
  /**
   * 
   * @param items 
   */
  public void addNewSourceItem(ArrayList<E> items) {
    
    this.sourceListPanel.addItems(items);
    
    this.sourceListPanel.selectLastInsertedRow();
  }

  /** This method is called from within the constructor to
   * initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is
   * always regenerated by the Form Editor.
   */
  @SuppressWarnings("unchecked")
  // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
  private void initComponents() {

    sourcePanel = new javax.swing.JPanel();
    detailPanel = new javax.swing.JPanel();

    sourcePanel.setLayout(new javax.swing.BoxLayout(sourcePanel, javax.swing.BoxLayout.LINE_AXIS));

    detailPanel.setLayout(new javax.swing.BoxLayout(detailPanel, javax.swing.BoxLayout.Y_AXIS));

    javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
    this.setLayout(layout);
    layout.setHorizontalGroup(
      layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
      .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
        .addComponent(sourcePanel, javax.swing.GroupLayout.DEFAULT_SIZE, 141, Short.MAX_VALUE)
        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
        .addComponent(detailPanel, javax.swing.GroupLayout.DEFAULT_SIZE, 126, Short.MAX_VALUE))
    );
    layout.setVerticalGroup(
      layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
      .addComponent(sourcePanel, javax.swing.GroupLayout.DEFAULT_SIZE, 135, Short.MAX_VALUE)
      .addComponent(detailPanel, javax.swing.GroupLayout.DEFAULT_SIZE, 135, Short.MAX_VALUE)
    );
  }// </editor-fold>//GEN-END:initComponents
  // Variables declaration - do not modify//GEN-BEGIN:variables
  private javax.swing.JPanel detailPanel;
  private javax.swing.JPanel sourcePanel;
  // End of variables declaration//GEN-END:variables
  private GenericDynamicItemsListPanel<E> sourceListPanel;
  private GenericDetailPanel<E> detailItemPanel;
  private E selectedItem;
  private ArrayList<TargetItemSelectedObserver> targetItemSelecteds = new ArrayList<TargetItemSelectedObserver>();
}
