/*
 * DocumentSlaveJournalPanel.java
 *
 * Created on 21-mar-2011, 11:26:38
 */
package scimat.gui.components.slavepanel;

import java.util.ArrayList;
import scimat.gui.components.ErrorDialogManager;
import scimat.model.knowledgebase.entity.Document;
import scimat.model.knowledgebase.entity.Journal;
import scimat.model.knowledgebase.exception.KnowledgeBaseException;
import scimat.project.CurrentProject;
import scimat.project.observer.EntityObserver;
import scimat.project.observer.JournalRelationDocumentObserver;

/**
 *
 * @author mjcobo
 */
public class DocumentSlaveJournalPanel extends GenericOneSlaveItemPanel 
        implements JournalRelationDocumentObserver, EntityObserver<Journal> {

  /** Creates new form DocumentSlaveJournalPanel */
  public DocumentSlaveJournalPanel() {

    initComponents();

    CurrentProject.getInstance().getKbObserver().addJournalRelationDocumentsObserver(this);
    CurrentProject.getInstance().getKbObserver().addJournalObserver(this);
  }

  /**
   *
   */
  private void refresh() {

    if (this.journal != null) {

      this.sourceTextField.setText(this.journal.getSource());
      fireSlaveItemObserver(true);

    } else {

      this.sourceTextField.setText("");
      fireSlaveItemObserver(false);
    }
  }

  /**
   *
   */
  public void setMasterItem(Document document) {

    this.document = document;

    try {

      if (this.document != null) {

        relationChanged();

      } else {

        this.journal = null;
        refresh();

      }

    } catch (KnowledgeBaseException e) {
    
      ErrorDialogManager.getInstance().showException(e);

    }
  }

  /**
   *
   */
  public void relationChanged() throws KnowledgeBaseException {

    if (this.document != null) {
    
      this.journal = CurrentProject.getInstance().getFactoryDAO().getDocumentDAO().getJournal(this.document.getDocumentID());
      
    } else {
    
      this.journal = null;
    }
    
    refresh();
  }

  /**
   *
   * @param items
   * @throws KnowledgeBaseException
   */
  public void entityAdded(ArrayList<Journal> items) throws KnowledgeBaseException {
    // Do not do nothing
  }

  /**
   *
   * @param entity
   * @throws KnowledgeBaseException
   */
  public void entityRefresh() throws KnowledgeBaseException {

    if (this.journal != null) {

      this.journal = CurrentProject.getInstance().getFactoryDAO().getJournalDAO().getJournal(this.document.getDocumentID());
      refresh();
    }
  }

  /**
   *
   * @param items
   * @throws KnowledgeBaseException
   */
  public void entityRemoved(ArrayList<Journal> items) throws KnowledgeBaseException {
    // Do not do nothing
  }

  /**
   *
   * @param items
   * @throws KnowledgeBaseException
   */
  public void entityUpdated(ArrayList<Journal> items) throws KnowledgeBaseException {

    int position;

    if (this.journal != null) {

      position = items.indexOf(this.journal);

      if (position != -1) {

        this.journal = items.get(position);
      }
    }
  }

  /** This method is called from within the constructor to
   * initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is
   * always regenerated by the Form Editor.
   */
  @SuppressWarnings("unchecked")
  // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
  private void initComponents() {

    sourceDescriptionLabel = new javax.swing.JLabel();
    sourceTextField = new javax.swing.JTextField();

    sourceDescriptionLabel.setText("Source:");

    sourceTextField.setEditable(false);

    javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
    this.setLayout(layout);
    layout.setHorizontalGroup(
      layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
      .addGroup(layout.createSequentialGroup()
        .addComponent(sourceDescriptionLabel)
        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
        .addComponent(sourceTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 263, Short.MAX_VALUE))
    );
    layout.setVerticalGroup(
      layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
      .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
        .addComponent(sourceDescriptionLabel)
        .addComponent(sourceTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
    );
  }// </editor-fold>//GEN-END:initComponents
  // Variables declaration - do not modify//GEN-BEGIN:variables
  private javax.swing.JLabel sourceDescriptionLabel;
  private javax.swing.JTextField sourceTextField;
  // End of variables declaration//GEN-END:variables
  private Document document = null;
  private Journal journal = null;
}
