/*
 * WordSlaveWordGroupPanel.java
 *
 * Created on 21-mar-2011, 11:34:59
 */
package scimat.gui.components.slavepanel;

import java.util.ArrayList;
import scimat.gui.components.ErrorDialogManager;
import scimat.model.knowledgebase.entity.Word;
import scimat.model.knowledgebase.entity.WordGroup;
import scimat.model.knowledgebase.exception.KnowledgeBaseException;
import scimat.project.CurrentProject;
import scimat.project.observer.EntityObserver;
import scimat.project.observer.WordGroupRelationWordObserver;

/**
 *
 * @author mjcobo
 */
public class WordSlaveWordGroupPanel extends GenericOneSlaveItemPanel
        implements WordGroupRelationWordObserver, EntityObserver<WordGroup> {

  /** Creates new form WordSlaveWordGroupPanel */
  public WordSlaveWordGroupPanel() {
    initComponents();

    CurrentProject.getInstance().getKbObserver().addWordGroupRelationWordsObserver(this);
    CurrentProject.getInstance().getKbObserver().addWordGroupObserver(this);
  }

  /**
   *
   */
  private void refresh() {

    if (this.wordGroup != null) {

      this.groupNameTextField.setText(this.wordGroup.getGroupName());
      this.stopGroupCheckBox.setSelected(this.wordGroup.isStopGroup());
      fireSlaveItemObserver(true);

    } else {

      this.groupNameTextField.setText("");
      this.stopGroupCheckBox.setSelected(false);
    }
  }

  /**
   *
   */
  public void setMasterItem(Word word) {

    this.word = word;

    try {

      if (this.word != null) {

        relationChanged();

      } else {

        this.wordGroup = null;
        refresh();

      }

    } catch (KnowledgeBaseException e) {
    
      ErrorDialogManager.getInstance().showException(e);

    }
  }

  /**
   *
   */
  public void relationChanged() throws KnowledgeBaseException {

    if (this.word != null) {
      
      this.wordGroup = CurrentProject.getInstance().getFactoryDAO().getWordDAO().getWordGroup(this.word.getWordID());
      
    } else {
    
      this.wordGroup = null;
    }
    
    refresh();
  }

  /**
   *
   * @param items
   * @throws KnowledgeBaseException
   */
  public void entityAdded(ArrayList<WordGroup> items) throws KnowledgeBaseException {
    // Do not do nothing
  }

  /**
   *
   * @param entity
   * @throws KnowledgeBaseException
   */
  public void entityRefresh() throws KnowledgeBaseException {

    if (this.wordGroup != null) {

      this.wordGroup = CurrentProject.getInstance().getFactoryDAO().getWordGroupDAO().getWordGroup(this.wordGroup.getWordGroupID());
      refresh();
    }
  }

  /**
   *
   * @param items
   * @throws KnowledgeBaseException
   */
  public void entityRemoved(ArrayList<WordGroup> items) throws KnowledgeBaseException {
    // Do not do nothing
  }

  /**
   *
   * @param items
   * @throws KnowledgeBaseException
   */
  public void entityUpdated(ArrayList<WordGroup> items) throws KnowledgeBaseException {

    int position;

    if (this.wordGroup != null) {

      position = items.indexOf(this.wordGroup);

      if (position != -1) {

        this.wordGroup = items.get(position);
      }
    }
  }

  /** This method is called from within the constructor to
   * initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is
   * always regenerated by the Form Editor.
   */
  @SuppressWarnings("unchecked")
  // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
  private void initComponents() {

    groupNameDescriptionLabel = new javax.swing.JLabel();
    groupNameTextField = new javax.swing.JTextField();
    stopGroupCheckBox = new javax.swing.JCheckBox();

    groupNameDescriptionLabel.setText("Group name:");

    groupNameTextField.setEditable(false);

    stopGroupCheckBox.setText("is stop group?");
    stopGroupCheckBox.setEnabled(false);

    javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
    this.setLayout(layout);
    layout.setHorizontalGroup(
      layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
      .addGroup(layout.createSequentialGroup()
        .addComponent(groupNameDescriptionLabel)
        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
        .addComponent(groupNameTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 245, Short.MAX_VALUE))
      .addGroup(layout.createSequentialGroup()
        .addComponent(stopGroupCheckBox)
        .addContainerGap())
    );
    layout.setVerticalGroup(
      layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
      .addGroup(layout.createSequentialGroup()
        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
          .addComponent(groupNameDescriptionLabel)
          .addComponent(groupNameTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
        .addComponent(stopGroupCheckBox))
    );
  }// </editor-fold>//GEN-END:initComponents
  // Variables declaration - do not modify//GEN-BEGIN:variables
  private javax.swing.JLabel groupNameDescriptionLabel;
  private javax.swing.JTextField groupNameTextField;
  private javax.swing.JCheckBox stopGroupCheckBox;
  // End of variables declaration//GEN-END:variables
  private Word word = null;
  private WordGroup wordGroup = null;
}
